@@ -166,7 +166,7 @@ class PlotAccessor:
         outline_color: str | list[float] = "#000000ff",
         outline_alpha: float | int = 0.0,
         cmap: Colormap | str | None = None,
-        norm: bool | Normalize = False,
+        norm: Normalize | None = None,
         scale: float | int = 1.0,
         method: str | None = None,
         table_name: str | None = None,
@@ -227,8 +227,15 @@ class PlotAccessor:
             Name of the table containing the color(s) columns. If one name is given than the table is used for each
             spatial element to be plotted if the table annotates it. If you want to use different tables for particular
             elements, as specified under element.
+
         **kwargs : Any
-            Additional arguments to be passed to cmap and norm.
+            Additional arguments for customization. This can include:
+
+            datashader_reduction : Literal[
+                "sum", "mean", "any", "count", "std", "var", "max", "min"
+            ], default: "sum"
+                Reduction method for datashader when coloring by continuous values. Defaults to 'sum'.
+
 
         Notes
         -----
@@ -241,6 +248,13 @@ class PlotAccessor:
         sd.SpatialData
             The modified SpatialData object with the rendered shapes.
         """
+        # TODO add Normalize object in tutorial notebook and point to that notebook here
+        if "vmin" in kwargs or "vmax" in kwargs:
+            warnings.warn(
+                "`vmin` and `vmax` are deprecated. Pass matplotlib `Normalize` object to norm instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
         params_dict = _validate_shape_render_params(
             self._sdata,
             element=element,
@@ -257,19 +271,18 @@ class PlotAccessor:
             scale=scale,
             table_name=table_name,
             method=method,
+            ds_reduction=kwargs.get("datashader_reduction", None),
         )
 
         sdata = self._copy()
         sdata = _verify_plotting_tree(sdata)
         n_steps = len(sdata.plotting_tree.keys())
         outline_params = _set_outline(outline_alpha > 0, outline_width, outline_color)
-
         for element, param_values in params_dict.items():
             cmap_params = _prepare_cmap_norm(
                 cmap=cmap,
                 norm=norm,
                 na_color=params_dict[element]["na_color"],  # type: ignore[arg-type]
-                **kwargs,
             )
             sdata.plotting_tree[f"{n_steps+1}_render_shapes"] = ShapesRenderParams(
                 element=element,
@@ -285,7 +298,8 @@ class PlotAccessor:
                 transfunc=kwargs.get("transfunc", None),
                 table_name=param_values["table_name"],
                 zorder=n_steps,
-                method=method,
+                method=param_values["method"],
+                ds_reduction=param_values["ds_reduction"],
             )
             n_steps += 1
 
@@ -301,7 +315,7 @@ class PlotAccessor:
         palette: list[str] | str | None = None,
         na_color: ColorLike | None = "default",
         cmap: Colormap | str | None = None,
-        norm: None | Normalize = None,
+        norm: Normalize | None = None,
         size: float | int = 1.0,
         method: str | None = None,
         table_name: str | None = None,
@@ -355,14 +369,27 @@ class PlotAccessor:
             Name of the table containing the color(s) columns. If one name is given than the table is used for each
             spatial element to be plotted if the table annotates it. If you want to use different tables for particular
             elements, as specified under element.
-        kwargs
-            Additional arguments to be passed to cmap and norm.
+
+        **kwargs : Any
+            Additional arguments for customization. This can include:
+
+            datashader_reduction : Literal[
+                "sum", "mean", "any", "count", "std", "var", "max", "min"
+            ], default: "sum"
+                Reduction method for datashader when coloring by continuous values. Defaults to 'sum'.
 
         Returns
         -------
         sd.SpatialData
             The modified SpatialData object with the rendered shapes.
         """
+        # TODO add Normalize object in tutorial notebook and point to that notebook here
+        if "vmin" in kwargs or "vmax" in kwargs:
+            warnings.warn(
+                "`vmin` and `vmax` are deprecated. Pass matplotlib `Normalize` object to norm instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
         params_dict = _validate_points_render_params(
             self._sdata,
             element=element,
@@ -375,6 +402,7 @@ class PlotAccessor:
             norm=norm,
             size=size,
             table_name=table_name,
+            ds_reduction=kwargs.get("datashader_reduction", None),
         )
 
         if method is not None:
@@ -392,7 +420,6 @@ class PlotAccessor:
                 cmap=cmap,
                 norm=norm,
                 na_color=param_values["na_color"],  # type: ignore[arg-type]
-                **kwargs,
             )
             sdata.plotting_tree[f"{n_steps+1}_render_points"] = PointsRenderParams(
                 element=element,
@@ -407,6 +434,7 @@ class PlotAccessor:
                 table_name=param_values["table_name"],
                 zorder=n_steps,
                 method=method,
+                ds_reduction=param_values["ds_reduction"],
             )
             n_steps += 1
 
@@ -422,7 +450,6 @@ class PlotAccessor:
         na_color: ColorLike | None = "default",
         palette: list[str] | str | None = None,
         alpha: float | int = 1.0,
-        percentiles_for_norm: tuple[float, float] | None = None,
         scale: str | None = None,
         **kwargs: Any,
     ) -> sd.SpatialData:
@@ -457,8 +484,6 @@ class PlotAccessor:
             Palette to color images. The number of palettes should be equal to the number of channels.
         alpha : float | int, default 1.0
             Alpha value for the images. Must be a numeric between 0 and 1.
-        percentiles_for_norm : tuple[float, float] | None
-            Optional pair of floats (pmin < pmax, 0-100) which will be used for quantile normalization.
         scale : str | None
             Influences the resolution of the rendering. Possibilities include:
                 1) `None` (default): The image is rasterized to fit the canvas size. For
@@ -476,6 +501,13 @@ class PlotAccessor:
         sd.SpatialData
             The SpatialData object with the rendered images.
         """
+        # TODO add Normalize object in tutorial notebook and point to that notebook here
+        if "vmin" in kwargs or "vmax" in kwargs:
+            warnings.warn(
+                "`vmin` and `vmax` are deprecated. Pass matplotlib `Normalize` object to norm instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
         params_dict = _validate_image_render_params(
             self._sdata,
             element=element,
@@ -486,7 +518,6 @@ class PlotAccessor:
             cmap=cmap,
             norm=norm,
             scale=scale,
-            percentiles_for_norm=percentiles_for_norm,
         )
 
         sdata = self._copy()
@@ -494,12 +525,7 @@ class PlotAccessor:
         n_steps = len(sdata.plotting_tree.keys())
 
         for element, param_values in params_dict.items():
-            # cmap_params = _prepare_cmap_norm(
-            #     cmap=params_dict[element]["cmap"],
-            #     norm=norm,
-            #     na_color=params_dict[element]["na_color"],  # type: ignore[arg-type]
-            #     **kwargs,
-            # )
+
             cmap_params: list[CmapParams] | CmapParams
             if isinstance(cmap, list):
                 cmap_params = [
@@ -507,7 +533,6 @@ class PlotAccessor:
                         cmap=c,
                         norm=norm,
                         na_color=param_values["na_color"],
-                        **kwargs,
                     )
                     for c in cmap
                 ]
@@ -525,7 +550,6 @@ class PlotAccessor:
                 cmap_params=cmap_params,
                 palette=param_values["palette"],
                 alpha=param_values["alpha"],
-                percentiles_for_norm=param_values["percentiles_for_norm"],
                 scale=param_values["scale"],
                 zorder=n_steps,
             )
@@ -608,6 +632,13 @@ class PlotAccessor:
         -------
         None
         """
+        # TODO add Normalize object in tutorial notebook and point to that notebook here
+        if "vmin" in kwargs or "vmax" in kwargs:
+            warnings.warn(
+                "`vmin` and `vmax` are deprecated. Pass matplotlib `Normalize` object to norm instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
         params_dict = _validate_label_render_params(
             self._sdata,
             element=element,
@@ -633,7 +664,6 @@ class PlotAccessor:
                 cmap=cmap,
                 norm=norm,
                 na_color=param_values["na_color"],  # type: ignore[arg-type]
-                **kwargs,
             )
             sdata.plotting_tree[f"{n_steps+1}_render_labels"] = LabelsRenderParams(
                 element=element,
