@@ -9,7 +9,7 @@ import dask
 import datashader as ds
 import geopandas as gpd
 import matplotlib
-import matplotlib.transforms as mtransforms
+import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import scanpy as sc
@@ -22,7 +22,6 @@ from scanpy._settings import settings as sc_settings
 from spatialdata import get_extent
 from spatialdata.models import PointsModel, get_table_keys
 from spatialdata.transformations import (
-    get_transformation,
     set_transformation,
 )
 
@@ -38,16 +37,20 @@ from spatialdata_plot.pl.render_params import (
 )
 from spatialdata_plot.pl.utils import (
     _ax_show_and_transform,
+    _create_image_from_datashader_result,
+    _datashader_aggregate_with_function,
+    _datshader_get_how_kw_for_spread,
     _decorate_axs,
     _get_collection_shape,
     _get_colors_for_categorical_obs,
+    _get_extent_and_range_for_datashader_canvas,
     _get_linear_colormap,
     _is_coercable_to_float,
     _map_color_seg,
     _maybe_set_colors,
     _mpl_ax_contains_elements,
     _multiscale_to_spatial_image,
-    _normalize,
+    _prepare_transformation,
     _rasterize_if_necessary,
     _set_color_source_vec,
     to_hex,
@@ -128,7 +131,6 @@ def _render_shapes(
         shapes = shapes.reset_index()
         color_source_vector = color_source_vector[mask]
         color_vector = color_vector[mask]
-    shapes = gpd.GeoDataFrame(shapes, geometry="geometry")
 
     # Using dict.fromkeys here since set returns in arbitrary order
     # remove the color of NaN values, else it might be assigned to a category
@@ -147,35 +149,33 @@ def _render_shapes(
         colorbar = False if col_for_color is None else legend_params.colorbar
 
     # Apply the transformation to the PatchCollection's paths
-    trans = get_transformation(sdata_filt.shapes[element], get_all=True)[coordinate_system]
-    affine_trans = trans.to_affine_matrix(input_axes=("x", "y"), output_axes=("x", "y"))
-    trans = mtransforms.Affine2D(matrix=affine_trans)
+    trans, _ = _prepare_transformation(sdata_filt.shapes[element], coordinate_system)
 
     shapes = gpd.GeoDataFrame(shapes, geometry="geometry")
 
     # Determine which method to use for rendering
     method = render_params.method
+
     if method is None:
         method = "datashader" if len(shapes) > 10000 else "matplotlib"
-    elif method not in ["matplotlib", "datashader"]:
-        raise ValueError("Method must be either 'matplotlib' or 'datashader'.")
+
     if method != "matplotlib":
         # we only notify the user when we switched away from matplotlib
-        logger.info(f"Using '{method}' as plotting backend.")
+        logger.info(
+            f"Using '{method}' backend with '{render_params.ds_reduction}' as reduction"
+            " method to speed up plotting. Depending on the reduction method, the value"
+            " range of the plot might change. Set method to 'matplotlib' do disable"
+            " this behaviour."
+        )
 
     if method == "datashader":
-        trans = mtransforms.Affine2D(matrix=affine_trans) + ax.transData
+        trans += ax.transData
 
-        extent = get_extent(sdata.shapes[element])
-        x_ext = extent["x"][1]
-        y_ext = extent["y"][1]
-        x_range = [0, x_ext]
-        y_range = [0, y_ext]
-        # round because we need integers
-        plot_width = int(np.round(x_range[1] - x_range[0]))
-        plot_height = int(np.round(y_range[1] - y_range[0]))
+        plot_width, plot_height, x_ext, y_ext, factor = _get_extent_and_range_for_datashader_canvas(
+            sdata_filt.shapes[element], coordinate_system, ax, fig_params
+        )
 
-        cvs = ds.Canvas(plot_width=plot_width, plot_height=plot_height, x_range=x_range, y_range=y_range)
+        cvs = ds.Canvas(plot_width=plot_width, plot_height=plot_height, x_range=x_ext, y_range=y_ext)
 
         _geometry = shapes["geometry"]
         is_point = _geometry.type == "Point"
@@ -192,18 +192,45 @@ def _render_shapes(
             )
         # Render shapes with datashader
         color_by_categorical = col_for_color is not None and color_source_vector is not None
-        aggregate_with_sum = None
+        aggregate_with_reduction = None
         if col_for_color is not None and (render_params.groups is None or len(render_params.groups) > 1):
             if color_by_categorical:
                 agg = cvs.polygons(
                     sdata_filt.shapes[element], geometry="geometry", agg=ds.by(col_for_color, ds.count())
                 )
             else:
-                agg = cvs.polygons(sdata_filt.shapes[element], geometry="geometry", agg=ds.sum(column=col_for_color))
+                reduction_name = render_params.ds_reduction if render_params.ds_reduction is not None else "mean"
+                logger.info(
+                    f'Using the datashader reduction "{reduction_name}". "max" will give an output very close '
+                    "to the matplotlib result."
+                )
+                agg = _datashader_aggregate_with_function(
+                    render_params.ds_reduction, cvs, sdata_filt.shapes[element], col_for_color, "shapes"
+                )
                 # save min and max values for drawing the colorbar
-                aggregate_with_sum = (agg.min(), agg.max())
+                aggregate_with_reduction = (agg.min(), agg.max())
         else:
             agg = cvs.polygons(sdata_filt.shapes[element], geometry="geometry", agg=ds.count())
+        # render outlines if needed
+        if (render_outlines := render_params.outline_alpha) > 0:
+            agg_outlines = cvs.line(
+                sdata_filt.shapes[element],
+                geometry="geometry",
+                line_width=render_params.outline_params.linewidth,
+            )
+
+        if norm.vmin is not None or norm.vmax is not None:
+            norm.vmin = np.min(agg) if norm.vmin is None else norm.vmin
+            norm.vmax = np.max(agg) if norm.vmax is None else norm.vmax
+            norm.clip = True  # NOTE: mpl currently behaves like clip is always True
+            if norm.vmin == norm.vmax:
+                # data is mapped to 0
+                agg = agg - agg
+            else:
+                agg = (agg - norm.vmin) / (norm.vmax - norm.vmin)
+                if norm.clip:
+                    agg = np.maximum(agg, 0)
+                    agg = np.minimum(agg, 1)
 
         color_key = (
             [x[:-2] for x in color_vector.categories.values]
@@ -212,26 +239,73 @@ def _render_shapes(
             else None
         )
 
-        ds_result = (
-            ds.tf.shade(
+        if color_by_categorical or col_for_color is None:
+            ds_cmap = None
+            if color_vector is not None:
+                ds_cmap = color_vector[0]
+                if isinstance(ds_cmap, str) and ds_cmap[0] == "#":
+                    ds_cmap = ds_cmap[:-2]
+
+            ds_result = ds.tf.shade(
                 agg,
-                cmap=color_vector[0][:-2],
+                cmap=ds_cmap,
                 color_key=color_key,
-                min_alpha=np.min([150, render_params.fill_alpha * 255]),
+                min_alpha=np.min([254, render_params.fill_alpha * 255]),
+                how="linear",
             )
-            if color_by_categorical or col_for_color is None
-            else ds.tf.shade(
+        elif aggregate_with_reduction is not None:  # to shut up mypy
+            ds_cmap = render_params.cmap_params.cmap
+            # in case all elements have the same value X: we render them using cmap(0.0),
+            # using an artificial "span" of [X, X + 1] for the color bar
+            # else: all elements would get alpha=0 and the color bar would have a weird range
+            if aggregate_with_reduction[0] == aggregate_with_reduction[1]:
+                ds_cmap = matplotlib.colors.to_hex(render_params.cmap_params.cmap(0.0), keep_alpha=False)
+                aggregate_with_reduction = (aggregate_with_reduction[0], aggregate_with_reduction[0] + 1)
+
+            ds_result = ds.tf.shade(
                 agg,
-                cmap=render_params.cmap_params.cmap,
+                cmap=ds_cmap,
+                how="linear",
+                min_alpha=np.min([254, render_params.fill_alpha * 255]),
             )
+
+        # shade outlines if needed
+        outline_color = render_params.outline_params.outline_color
+        if isinstance(outline_color, str) and outline_color.startswith("#") and len(outline_color) == 9:
+            logger.info(
+                "alpha component of given RGBA value for outline color is discarded, because outline_alpha"
+                " takes precedent."
+            )
+            outline_color = outline_color[:-2]
+
+        if render_outlines:
+            ds_outlines = ds.tf.shade(
+                agg_outlines,
+                cmap=outline_color,
+                min_alpha=np.min([254, render_params.outline_alpha * 255]),
+                how="linear",
+            )
+
+        rgba_image, trans_data = _create_image_from_datashader_result(ds_result, factor, ax)
+        _cax = _ax_show_and_transform(
+            rgba_image, trans_data, ax, zorder=render_params.zorder, alpha=render_params.fill_alpha
         )
-        rgba_image = np.transpose(ds_result.to_numpy().base, (0, 1, 2))
-        _cax = ax.imshow(rgba_image, cmap=palette, zorder=render_params.zorder)
-        _cax.set_transform(trans)
-        cax = ax.add_image(_cax)
-        if aggregate_with_sum is not None:
+        # render outline image if needed
+        if render_outlines:
+            rgba_image, trans_data = _create_image_from_datashader_result(ds_outlines, factor, ax)
+            _ax_show_and_transform(
+                rgba_image, trans_data, ax, zorder=render_params.zorder, alpha=render_params.outline_alpha
+            )
+
+        cax = None
+        if aggregate_with_reduction is not None:
+            vmin = aggregate_with_reduction[0].values if norm.vmin is None else norm.vmin
+            vmax = aggregate_with_reduction[1].values if norm.vmin is None else norm.vmax
+            if (norm.vmin is not None or norm.vmax is not None) and norm.vmin == norm.vmax:
+                vmin = norm.vmin
+                vmax = norm.vmin + 1
             cax = ScalarMappable(
-                norm=matplotlib.colors.Normalize(vmin=aggregate_with_sum[0], vmax=aggregate_with_sum[1]),
+                norm=matplotlib.colors.Normalize(vmin=vmin, vmax=vmax),
                 cmap=render_params.cmap_params.cmap,
             )
 
@@ -255,9 +329,13 @@ def _render_shapes(
         for path in _cax.get_paths():
             path.vertices = trans.transform(path.vertices)
 
-    # Sets the limits of the colorbar to the values instead of [0, 1]
-    if not norm and not values_are_categorical:
-        _cax.set_clim(min(color_vector), max(color_vector))
+    if not values_are_categorical:
+        # If the user passed a Normalize object with vmin/vmax we'll use those,
+        # # if not we'll use the min/max of the color_vector
+        _cax.set_clim(
+            vmin=render_params.cmap_params.norm.vmin or min(color_vector),
+            vmax=render_params.cmap_params.norm.vmax or max(color_vector),
+        )
 
     if len(set(color_vector)) != 1 or list(set(color_vector))[0] != to_hex(render_params.cmap_params.na_color):
         # necessary in case different shapes elements are annotated with one table
@@ -391,60 +469,68 @@ def _render_points(
     if color_source_vector is None and render_params.transfunc is not None:
         color_vector = render_params.transfunc(color_vector)
 
-    trans = get_transformation(sdata.points[element], get_all=True)[coordinate_system]
-    affine_trans = trans.to_affine_matrix(input_axes=("x", "y"), output_axes=("x", "y"))
-    trans = mtransforms.Affine2D(matrix=affine_trans) + ax.transData
+    _, trans_data = _prepare_transformation(sdata.points[element], coordinate_system, ax)
 
     norm = copy(render_params.cmap_params.norm)
 
     method = render_params.method
+
     if method is None:
         method = "datashader" if len(points) > 10000 else "matplotlib"
-    elif method not in ["matplotlib", "datashader"]:
-        raise ValueError("Method must be either 'matplotlib' or 'datashader'.")
+
+    if method != "matplotlib":
+        # we only notify the user when we switched away from matplotlib
+        logger.info(
+            f"Using '{method}' backend with '{render_params.ds_reduction}' as reduction"
+            " method to speed up plotting. Depending on the reduction method, the value"
+            " range of the plot might change. Set method to 'matplotlib' do disable"
+            " this behaviour."
+        )
 
     if method == "datashader":
         # NOTE: s in matplotlib is in units of points**2
-        px = int(np.round(np.sqrt(render_params.size)))
+        # use dpi/100 as a factor for cases where dpi!=100
+        px = int(np.round(np.sqrt(render_params.size) * (fig_params.fig.dpi / 100)))
 
-        extent = get_extent(sdata_filt.points[element], coordinate_system=coordinate_system)
-        x_ext = [min(0, extent["x"][0]), extent["x"][1]]
-        y_ext = [min(0, extent["y"][0]), extent["y"][1]]
-        previous_xlim = ax.get_xlim()
-        previous_ylim = ax.get_ylim()
-        # increase range if sth larger was rendered before
-        if _mpl_ax_contains_elements(ax):
-            x_ext = [min(x_ext[0], previous_xlim[0]), max(x_ext[1], previous_xlim[1])]
-            y_ext = (
-                [
-                    min(y_ext[0], previous_ylim[1]),
-                    max(y_ext[1], previous_ylim[0]),
-                ]
-                if ax.yaxis_inverted()
-                else [
-                    min(y_ext[0], previous_ylim[0]),
-                    max(y_ext[1], previous_ylim[1]),
-                ]
-            )
-        # round because we need integers
-        plot_width = int(np.round(x_ext[1] - x_ext[0]))
-        plot_height = int(np.round(y_ext[1] - y_ext[0]))
+        plot_width, plot_height, x_ext, y_ext, factor = _get_extent_and_range_for_datashader_canvas(
+            sdata_filt.points[element], coordinate_system, ax, fig_params
+        )
 
         # use datashader for the visualization of points
         cvs = ds.Canvas(plot_width=plot_width, plot_height=plot_height, x_range=x_ext, y_range=y_ext)
 
         color_by_categorical = col_for_color is not None and points[col_for_color].values.dtype == object
-        aggregate_with_sum = None
+        aggregate_with_reduction = None
         if col_for_color is not None and (render_params.groups is None or len(render_params.groups) > 1):
             if color_by_categorical:
                 agg = cvs.points(sdata_filt.points[element], "x", "y", agg=ds.by(col_for_color, ds.count()))
             else:
-                agg = cvs.points(sdata_filt.points[element], "x", "y", agg=ds.sum(column=col_for_color))
+                reduction_name = render_params.ds_reduction if render_params.ds_reduction is not None else "sum"
+                logger.info(
+                    f'Using the datashader reduction "{reduction_name}". "max" will give an output very close '
+                    "to the matplotlib result."
+                )
+                agg = _datashader_aggregate_with_function(
+                    render_params.ds_reduction, cvs, sdata_filt.points[element], col_for_color, "points"
+                )
                 # save min and max values for drawing the colorbar
-                aggregate_with_sum = (agg.min(), agg.max())
+                aggregate_with_reduction = (agg.min(), agg.max())
         else:
             agg = cvs.points(sdata_filt.points[element], "x", "y", agg=ds.count())
 
+        if norm.vmin is not None or norm.vmax is not None:
+            norm.vmin = np.min(agg) if norm.vmin is None else norm.vmin
+            norm.vmax = np.max(agg) if norm.vmax is None else norm.vmax
+            norm.clip = True  # NOTE: mpl currently behaves like clip is always True
+            if norm.vmin == norm.vmax:
+                # data is mapped to 0
+                agg = agg - agg
+            else:
+                agg = (agg - norm.vmin) / (norm.vmax - norm.vmin)
+                if norm.clip:
+                    agg = np.maximum(agg, 0)
+                    agg = np.minimum(agg, 1)
+
         color_key = (
             list(color_vector.categories.values)
             if (type(color_vector) is pd.core.arrays.categorical.Categorical)
@@ -460,26 +546,45 @@ def _render_points(
         ):
             color_vector = np.asarray([x[:-2] for x in color_vector])
 
-        ds_result = (
-            ds.tf.shade(
+        if color_by_categorical or col_for_color is None:
+            ds_result = ds.tf.shade(
                 ds.tf.spread(agg, px=px),
-                rescale_discrete_levels=True,
                 cmap=color_vector[0],
                 color_key=color_key,
-                min_alpha=np.min([150, render_params.alpha * 255]),  # value 150 is arbitrarily chosen
+                min_alpha=np.min([254, render_params.alpha * 255]),
+                how="linear",
             )
-            if color_by_categorical or col_for_color is None
-            else ds.tf.shade(
-                ds.tf.spread(agg, px=px),
-                rescale_discrete_levels=True,
-                cmap=render_params.cmap_params.cmap,
+        else:
+            spread_how = _datshader_get_how_kw_for_spread(render_params.ds_reduction)
+            agg = ds.tf.spread(agg, px=px, how=spread_how)
+            aggregate_with_reduction = (agg.min(), agg.max())
+
+            ds_cmap = render_params.cmap_params.cmap
+            # in case all elements have the same value X: we render them using cmap(0.0),
+            # using an artificial "span" of [X, X + 1] for the color bar
+            # else: all elements would get alpha=0 and the color bar would have a weird range
+            if aggregate_with_reduction[0] == aggregate_with_reduction[1]:
+                ds_cmap = matplotlib.colors.to_hex(render_params.cmap_params.cmap(0.0), keep_alpha=False)
+                aggregate_with_reduction = (aggregate_with_reduction[0], aggregate_with_reduction[0] + 1)
+
+            ds_result = ds.tf.shade(
+                agg,
+                cmap=ds_cmap,
+                how="linear",
             )
-        )
-        rbga_image = np.transpose(ds_result.to_numpy().base, (0, 1, 2))
-        cax = ax.imshow(rbga_image, zorder=render_params.zorder, alpha=render_params.alpha)
-        if aggregate_with_sum is not None:
+
+        rgba_image, trans_data = _create_image_from_datashader_result(ds_result, factor, ax)
+        _ax_show_and_transform(rgba_image, trans_data, ax, zorder=render_params.zorder, alpha=render_params.alpha)
+
+        cax = None
+        if aggregate_with_reduction is not None:
+            vmin = aggregate_with_reduction[0].values if norm.vmin is None else norm.vmin
+            vmax = aggregate_with_reduction[1].values if norm.vmax is None else norm.vmax
+            if (norm.vmin is not None or norm.vmax is not None) and norm.vmin == norm.vmax:
+                vmin = norm.vmin
+                vmax = norm.vmin + 1
             cax = ScalarMappable(
-                norm=matplotlib.colors.Normalize(vmin=aggregate_with_sum[0], vmax=aggregate_with_sum[1]),
+                norm=matplotlib.colors.Normalize(vmin=vmin, vmax=vmax),
                 cmap=render_params.cmap_params.cmap,
             )
 
@@ -495,7 +600,7 @@ def _render_points(
             cmap=render_params.cmap_params.cmap,
             norm=norm,
             alpha=render_params.alpha,
-            transform=trans,
+            transform=trans_data,
             zorder=render_params.zorder,
         )
         cax = ax.add_collection(_cax)
@@ -593,21 +698,12 @@ def _render_images(
     if isinstance(render_params.cmap_params, list) and len(render_params.cmap_params) != n_channels:
         raise ValueError("If 'cmap' is provided, its length must match the number of channels.")
 
-    # prepare transformations
-    trans = get_transformation(img, get_all=True)[coordinate_system]
-    affine_trans = trans.to_affine_matrix(input_axes=("x", "y"), output_axes=("x", "y"))
-    trans = mtransforms.Affine2D(matrix=affine_trans)
-    trans_data = trans + ax.transData
+    _, trans_data = _prepare_transformation(img, coordinate_system, ax)
 
     # 1) Image has only 1 channel
     if n_channels == 1 and not isinstance(render_params.cmap_params, list):
         layer = img.sel(c=channels[0]).squeeze() if isinstance(channels[0], str) else img.isel(c=channels[0]).squeeze()
 
-        if render_params.percentiles_for_norm != (None, None):
-            layer = _normalize(
-                layer, pmin=render_params.percentiles_for_norm[0], pmax=render_params.percentiles_for_norm[1], clip=True
-            )
-
         if render_params.cmap_params.norm:  # type: ignore[attr-defined]
             layer = render_params.cmap_params.norm(layer)  # type: ignore[attr-defined]
 
@@ -623,20 +719,16 @@ def _render_images(
 
         _ax_show_and_transform(layer, trans_data, ax, cmap=cmap, zorder=render_params.zorder)
 
+        if legend_params.colorbar:
+            sm = plt.cm.ScalarMappable(cmap=cmap, norm=render_params.cmap_params.norm)
+            fig_params.fig.colorbar(sm, ax=ax)
+
     # 2) Image has any number of channels but 1
     else:
         layers = {}
         for ch_index, c in enumerate(channels):
             layers[c] = img.sel(c=c).copy(deep=True).squeeze()
 
-            if render_params.percentiles_for_norm != (None, None):
-                layers[c] = _normalize(
-                    layers[c],
-                    pmin=render_params.percentiles_for_norm[0],
-                    pmax=render_params.percentiles_for_norm[1],
-                    clip=True,
-                )
-
             if not isinstance(render_params.cmap_params, list):
                 if render_params.cmap_params.norm is not None:
                     layers[c] = render_params.cmap_params.norm(layers[c])
@@ -763,10 +855,7 @@ def _render_labels(
         # get instance id based on subsetted table
         instance_id = np.unique(table.obs[instance_key].values)
 
-    trans = get_transformation(label, get_all=True)[coordinate_system]
-    affine_trans = trans.to_affine_matrix(input_axes=("x", "y"), output_axes=("x", "y"))
-    trans = mtransforms.Affine2D(matrix=affine_trans)
-    trans_data = trans + ax.transData
+    _, trans_data = _prepare_transformation(label, coordinate_system, ax)
 
     color_source_vector, color_vector, categorical = _set_color_source_vec(
         sdata=sdata_filt,
@@ -855,7 +944,7 @@ def _render_labels(
         legend_fontweight=legend_params.legend_fontweight,
         legend_loc=legend_params.legend_loc,
         legend_fontoutline=legend_params.legend_fontoutline,
-        na_in_legend=legend_params.na_in_legend,
+        na_in_legend=legend_params.na_in_legend if groups is None else len(groups) == len(set(color_vector)),
         colorbar=legend_params.colorbar,
         scalebar_dx=scalebar_params.scalebar_dx,
         scalebar_units=scalebar_params.scalebar_units,
