@@ -58,25 +58,23 @@ class TestUtils(PlotTester, metaclass=PlotTesterMeta):
         from spatialdata_plot.pl.utils import set_zero_in_cmap_to_transparent
 
         # set up figure and modify the data to add 0s
-        fig, axs = plt.subplots(ncols=2, figsize=(6, 3))
-        table = sdata_blobs.table.copy()
-        x = table.X.todense()
-        x[:10, 0] = 0
-        table.X = x
-        sdata_blobs.tables["modified_table"] = table
+        _, axs = plt.subplots(nrows=1, ncols=2, layout="tight")
+        sdata_blobs.tables["table"].obs["my_var"] = list(range(len(sdata_blobs.tables["table"].obs)))
+        sdata_blobs.tables["table"].obs["my_var"] += 2  # shift the values to not have 0s
 
-        # create a new cmap with 0 as transparent
-        new_cmap = set_zero_in_cmap_to_transparent(cmap="plasma")
+        new_cmap = set_zero_in_cmap_to_transparent(cmap="viridis")
 
         # baseline img
-        sdata_blobs.pl.render_labels("blobs_labels", color="channel_0_sum", cmap="viridis", table="table").pl.show(
+        sdata_blobs.pl.render_labels("blobs_labels", color="my_var", cmap="viridis", table="table").pl.show(
             ax=axs[0], colorbar=False
         )
 
+        sdata_blobs.tables["table"].obs.iloc[8:12, 2] = 0
+
         # image with 0s as transparent, so some labels are "missing"
-        sdata_blobs.pl.render_labels(
-            "blobs_labels", color="channel_0_sum", cmap=new_cmap, table="modified_table"
-        ).pl.show(ax=axs[1], colorbar=False)
+        sdata_blobs.pl.render_labels("blobs_labels", color="my_var", cmap=new_cmap, table="table").pl.show(
+            ax=axs[1], colorbar=False
+        )
 
 
 @pytest.mark.parametrize(
